/* g++ -DLINUX examples/powermgr_example.cpp  -lcc-aux -pthread -ldl */
#include <PowerMgr.h>
#include <Battery.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

using namespace std;
using namespace CrossControl;
unsigned int suspendConfiguration;
//
// OS-independent Sleep function
//
#ifdef LINUX
static bool OSSleep(unsigned long delay_ms)
{
  struct timespec ts;
  ts.tv_sec = (delay_ms / 1000);
  ts.tv_nsec = (delay_ms - ts.tv_sec * 1000) * 1000000; /* mS */
  if (nanosleep(&ts, NULL) < 0)
  {
    return false;
  }
  return true;
}
#else
# include <windows.h>
static bool OSSleep(DWORD dwMilliseconds)
{
  ::Sleep(dwMilliseconds);
  return true;
}
#endif

//
// Variable used in the console application exit handler. It is set to false when the API handles the shutdown.
//
bool os_shutdown = true;



void test_powermgr(POWERMGRHANDLE pPowerMgr)
{
  if(!pPowerMgr)
    return;

  CrossControl::eErr err;
  PowerMgrConf setConfiguration;

  err = PowerMgr_getConfiguration(pPowerMgr, &setConfiguration);	
  if(err == ERR_SUCCESS)
  {
    if(setConfiguration == Normal)
    {
      cout << "Normal configuration used... Ctrl-C to exit." << endl;
    } 
    else if(setConfiguration == ApplicationControlled)
    {
      cout << "ApplicationControlled configuration used... Ctrl-C to exit." << endl;
    }
    else if(setConfiguration == BatterySuspend)
    {
      cout << "BatterySuspend configuration used... Ctrl-C to exit." << endl;
    } 
  } 
  else
  {
    cout << "PowerMgr_getConfiguration(pPowerMgr, &setConfiguration) Failed" << endl;		
  }



  //! [Calling getPowerMgrStatus]
  //! [Calling setAppReadyForSuspendOrShutdown]
  //! [Calling hasResumed]
  while(1)
  {
    OSSleep(500);

    PowerMgrStatus status;
    err = PowerMgr_getPowerMgrStatus(pPowerMgr, &status);
    if(err == ERR_SUCCESS)
    {
      switch(status)
      {
      case NoRequestsPending: // Wait until a PowerMgr request arrives...
        break; 

      case ShutdownPending:
        {
          // Shutdown by means of power button or on/off signal are caught here.
          os_shutdown = false;

          cout << "A shutdown request detected. App should now do what it needs to do before shutdown can be performed."  << endl;
          cout << "Press Enter when ready to shutdown... "  << endl;

          // Make sure to clear cin buffer before read
          std::cin.clear();
          std::cin.ignore(100,'\n');
          cin.get();
          cout << "Signalling that app is ready..."  << endl;
          err = PowerMgr_setAppReadyForSuspendOrShutdown(pPowerMgr);  
          if(err != ERR_SUCCESS)
          {
            cout << "Error(" << err << ") in function setAppReadyForSuspendOrShutdown: " << GetErrorStringA(err) << endl;
          }       
          return; //exit test appp
        }
      case SuspendPending: 
        {
          os_shutdown = false;

          cout << "A suspend request detected. App should now do what it needs to do before suspend can be performed."  << endl;
          cout << "Press Enter when ready to suspend... "  << endl;

          // Make sure to clear cin buffer before read
          std::cin.clear();
          std::cin.ignore(100,'\n');
          cin.get();
          cout << "Signalling that app is ready..."  << endl;
          err = PowerMgr_setAppReadyForSuspendOrShutdown(pPowerMgr);     
          if(err != ERR_SUCCESS)
          {
            cout << "Error(" << err << ") in function setAppReadyForSuspendOrShutdown: " << GetErrorStringA(err) << endl;
          }
        }
        break;

      default:
        cout << "Error: Invalid status returned from getPowerMgrStatus!"  << endl;
        break;
      }

      //Wait for resume after notifying that we are ready to suspend
      if(status == SuspendPending)
      {
        bool b = false;
        while(!b)
        {
          OSSleep(100);
          cout << "." << endl;

          err = PowerMgr_hasResumed(pPowerMgr, &b);   
          if(err != ERR_SUCCESS)
          {
            cout << "Error(" << err << ") in function hasResumed: " << GetErrorStringA(err) << endl;
          }
        }
        cout << "System is now resumed from suspend mode!"  << endl << 
          "Now we will soon re-register using the registerControlledSuspendOrShutDown function!"  << endl;

        // Expecting to get configuration Normal after resume from suspend 

        //! [Calling getConfiguration]
        CrossControl::PowerMgrConf conf;
        err = PowerMgr_getConfiguration(pPowerMgr, &conf);  
        if(err == ERR_SUCCESS)
        {
          switch (conf)
          {
          case Normal:
            cout << "PowerMgrConf is now: Normal" << endl; break;
          case ApplicationControlled: 
            cout << "PowerMgrConf is now: ApplicationControlled" << endl; break;
          case BatterySuspend: 
            cout << "PowerMgrConf is now: BatterySuspend" << endl; break;
          }
        }
        else
        {
          cout << "Error(" << err << ") in function getConfiguration: " << GetErrorStringA(err) << endl;
        }
        //! [Calling getConfiguration]


        // Re-register, do this as soon as possible after resume/startup  
        PowerMgr_registerControlledSuspendOrShutDown(pPowerMgr, setConfiguration);
        if(err == ERR_SUCCESS)
          cout << "Re-registered to powerMgr.  Ctrl-C to exit." << endl;
        else
          cout << "Error(" << err << ") in function registerControlledSuspendOrShutDown: " << GetErrorStringA(err) << endl;
      }
    }
    else
    {
      cout << "Error(" << err << ") in function getPowerMgrStatus: " << GetErrorStringA(err) << endl;
    }
  }
  //! [Calling hasResumed]
  //! [Calling setAppReadyForSuspendOrShutdown]
  //! [Calling getPowerMgrStatus]
}

//
// This function handles OS shutdown initiated from the OS itself and not from power button or on/off signal.
// 
void fnExit (void)
{
  if(os_shutdown)
  {
    cout << "Test application detected OS shutdown. Wait 5s before exit..." << endl;
    OSSleep(5000);
  }
}

int main(void)
{
  string input = "";
  //! [Calling Release]
  //! [Calling GetPowerMgr]
  //! [Calling registerControlledSuspendOrShutDown]
  CrossControl::eErr err;
  POWERMGRHANDLE pPowerMgr = ::GetPowerMgr();
  BATTERYHANDLE pBattery = ::GetBattery();

  assert(pPowerMgr);
  assert(pBattery);

  // Register a separate exit handler for the case where OS is initiating the shutdown. The Application must handle this case itself. 
  atexit(fnExit);

  bool bBatt = false;
  Battery_isBatteryPresent(pBattery, &bBatt);
  if(bBatt)   // Ask user wich configuration to use...
    cout << "Choose configuration to use, 0 - Normal, 1 - Application Controlled , 2 - Battery Suspend"  << endl;
  else
    cout << "Choose configuration to use, 0 - Normal, 1 - Application Controlled"  << endl;

  cin >> suspendConfiguration;
  Battery_release(pBattery);

  // Register that this application needs to delay suspend/shutdown
  // This should be done as soon as possible.
  // Then the app must poll getPowerMgrStatus() and allow the suspend/shutdown with setAppReadyForSuspendOrShutdown().
  // Depending on application design, this might be best handled in a separate thread.
  err = PowerMgr_registerControlledSuspendOrShutDown(pPowerMgr, (PowerMgrConf) suspendConfiguration);

  cout << "suspendConfiguration " << suspendConfiguration << endl;

  if(err == ERR_SUCCESS)
    cout << "Registered to powerMgr." << endl;
  else
    cout << "Error(" << err << ") in function registerControlledSuspendOrShutDown: " << GetErrorStringA(err) << endl;

  test_powermgr(pPowerMgr);

  PowerMgr_release(pPowerMgr);
  //! [Calling registerControlledSuspendOrShutDown]
  //! [Calling GetPowerMgr]
  //! [Calling Release]
}
