/************************************************
*
*  CROSSCONTROL AUX API
*
*  About.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description:
*  Get information about the CCpilot computer.
*
*  Use the function GetAbout() to get a handle to pass on to the other functions. 
*  Use the method About_release(ABOUTHANDLE) to return the handle.
*
*  Example Usage:
*  @include about_example.cpp
*
************************************************/

#ifndef CC_ABOUT_H_
#define CC_ABOUT_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* ABOUTHANDLE;

  /*!
  * Factory function that creates instances of the About object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return ABOUTHANDLE to an allocated About object. The returned
  * handle needs to be deallocated using the About_release(ABOUTHANDLE) method when it's
  * no longer needed.
  *
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling GetAbout
  */
  EXTERN_C
    CCAUXDLL_API
    ABOUTHANDLE
    CCAUXDLL_CALLING_CONV
    GetAbout(void);


  /*!
  * Delete the About object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC 
  *
  * @return -
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_release
  */
  EXTERN_C
    CCAUXDLL_API
    void 
    CCAUXDLL_CALLING_CONV
    About_release(ABOUTHANDLE);


  /*!
  * Get main PCB serial number.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainPCBSerial
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainPCBSerial (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get unit serial number.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getUnitSerial
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getUnitSerial (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get main PCB article number.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param buff Text output buffer.
  * @param length Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainPCBArt
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainPCBArt (ABOUTHANDLE, char* buff, int length);

  /*!
  * Get main manufacturing date.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainManufacturingDate
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainManufacturingDate (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get main hardware version (PCB revision).
  *
  * Supported Platform(s): XL, XM, XS, XA, VC 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainHWversion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainHWversion (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get main product revision.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainProdRev
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainProdRev (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get main product article number.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getMainProdArtNr
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getMainProdArtNr (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get number of ethernet connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param NrOfConnections Returns the number of connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfETHConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV 
    About_getNrOfETHConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get number of CAN connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param NrOfConnections Returns the number of connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfCANConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfCANConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get number of Video connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param NrOfConnections Returns the number of connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfVideoConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfVideoConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get number of USB connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param NrOfConnections Returns the number of connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfUSBConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfUSBConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get number of serial port (RS232) connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param NrOfConnections Returns the number of connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfSerialConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfSerialConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get number of digital I/O connections present.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param NrOfConnections Returns the number of input or input/output connections.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfDigIOConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfDigIOConnections (ABOUTHANDLE, unsigned char* NrOfConnections);

  /*!
  * Get Display module status. (Some product variants does not have a display)
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param available Is display available?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsDisplayAvailable
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsDisplayAvailable (ABOUTHANDLE, bool* available);

  /*!
  * Get Display TouchScreen status.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param available Is TouchScreen available?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsTouchScreenAvailable
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsTouchScreenAvailable (ABOUTHANDLE, bool* available);

  /*!
  * Get display resolution.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  * The display resolution will be returned in the format "1024x768"
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *  
  * Example Usage:
  * @snippet about_example.cpp Calling About_getDisplayResolution
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getDisplayResolution (ABOUTHANDLE, char* buff, int len);


  /*!
  * Get Add on PCB serial number.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getAddOnPCBSerial
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getAddOnPCBSerial (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get Add on PCB article number.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param buff Text output buffer.
  * @param length Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getAddOnPCBArt
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getAddOnPCBArt (ABOUTHANDLE, char* buff, int length);

  /*!
  * Get Add on manufacturing date.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getAddOnManufacturingDate
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getAddOnManufacturingDate (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get Add on hardware version.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param buff Text output buffer.
  * @param len Maximum length of the output buffer.
  * If the actual length of the data is greater, an error will be returned.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getAddOnHWversion
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getAddOnHWversion (ABOUTHANDLE, char* buff, int len);

  /*!
  * Get WLAN module mounting status.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param mounted Is module mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsWLANMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsWLANMounted (ABOUTHANDLE, bool* mounted);

  /*!
  * Get GPS module mounting status.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param mounted Is module mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsGPSMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsGPSMounted (ABOUTHANDLE, bool* mounted);

  /*!
  * Get GPRS module mounting status.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param mounted Is module mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsGPRSMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsGPRSMounted (ABOUTHANDLE, bool* mounted);

  /*!
  * Get BlueTooth module mounting status.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param mounted Is module mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsBTMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsBTMounted (ABOUTHANDLE, bool* mounted);

  /*!
  * Get the front hardware pcb revision in the format major.minor (e.g. 1.1). 
  *
  * Supported Platform(s): XA, XS
  *
  * @param major The major pcb revision.
  * @param minor The minor pcb revision.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getFrontPcbRev(ABOUTHANDLE, unsigned char* major, unsigned char* minor);

  /*!
  * Get IO Expander mounting status.
  *
  * Supported Platform(s): XA, XS
  *
  * @param mounted Is IO Expander mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsIOExpanderMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsIOExpanderMounted (ABOUTHANDLE, bool* mounted);


  /*!
  * Get Value for IO Expander
  *
  * Supported Platform(s): XA, XS
  *
  * @param value IO Expander value.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIOExpanderValue (ABOUTHANDLE, unsigned short* value);

  /*!
  * Get the status of the OS boot process. 
  * In Linux, drivers may be delay-loaded at start-up. 
  * If the application is started early in the boot-process, this function can be used to 
  * determine when full functionality can be obtained from the API/drivers.
  *
  * Supported Platform(s): XA, XS
  *
  * @param bootComplete Is the OS fully booted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_hasOsBooted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_hasOsBooted (ABOUTHANDLE, bool* bootComplete);

  /*!
  * Get Anybus mounting status.
  *
  * Supported Platform(s): XA, XS
  *
  * @param mounted Is Anybus mounted?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getIsAnybusMounted
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getIsAnybusMounted (ABOUTHANDLE, bool* mounted);

  /*!
  * Get number of configurable input connections present.
  *
  * Supported Platform(s): VC
  *
  * @param NrOfConnections Returns the number of inputs.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfCfgInConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    About_getNrOfCfgInConnections (ABOUTHANDLE, unsigned char* NrOfConnections);


  /*!
  * Get number of PWM Output connections present.
  *
  * Supported Platform(s): VC
  *
  * @param NrOfConnections Returns the number of outputs.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfPWMOutConnections
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    About_getNrOfPWMOutConnections (ABOUTHANDLE, unsigned char* NrOfConnections);


  /*!
  * Get number of configurable buttons.
  *
  * Supported Platform(s): VC
  *
  * @param numbuttons Number of configurable buttons.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *  
  * Example Usage:
  * @snippet about_example.cpp Calling About_getNrOfButtons
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getNrOfButtons (ABOUTHANDLE, int* numbuttons);


  /*!
  * Get User Eeprom data. The user eeprom holds 4096 bytes of data which are fully accessible.
  * Data is always read from position 0. 
  *
  * Supported Platform(s): XL, XM, XS, XA 
  * 
  * @param buff data buffer.
  * @param length data buffer length or number of data bytes to read.
  * @return error status. 0 = ERR_SUCCESS, otherwise error code.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_getUserEepromData(ABOUTHANDLE, char* buff, unsigned short length);

  /*!
  * Set User Eeprom data. The user eeprom holds 4096 bytes of data which are fully accessible.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  * 
  * @param startpos eeprom write start position.
  * @param buff data buffer.
  * @param length buffer length to write.
  * @return error status. 0 = ERR_SUCCESS, otherwise error code.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    About_setUserEepromData(ABOUTHANDLE, unsigned short startpos, const char* buff, unsigned short length);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_ABOUT_H_ */


