/************************************************
*
*  CROSSCONTROL AUX API
*
*  Adc.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Get current voltages from the built-in ADC
*
*  Use the function GetAdc() to get a handle to pass on to the other functions. 
*  Use the method Adc_release(ADCHANDLE) to return the handle.
*
*  Example Usage:
*  @include adc_example.cpp
*
************************************************/

#ifndef CC_ADC_H_
#define CC_ADC_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"
#include "CCAuxTypes.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* ADCHANDLE;

  /*!
  * Factory function that creates instances of the Adc object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return ADCHANDLE to an allocated Adc object. The returned
  * handle needs to be deallocated using the Adc_release(ADCHANDLE) method when it's
  * no longer needed.
  *
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet adc_example.cpp Calling GetAdc
  */
  EXTERN_C
    CCAUXDLL_API
    ADCHANDLE
    CCAUXDLL_CALLING_CONV
    GetAdc(void);

  /*!
  * Delete the ADC object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet adc_example.cpp Calling Adc_release
  */
  EXTERN_C
    CCAUXDLL_API
    void 
    CCAUXDLL_CALLING_CONV
    Adc_release(ADCHANDLE);

  /*!
  * Read measured voltage.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param selection The type of voltage to get.
  * @param value Voltage value in Volt. Can be undefined if return value is error code.
  * Not all values are supported on all platforms, ERR_NOT_SUPPORTED will indicate that.
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet adc_example.cpp Calling Adc_getVoltage
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Adc_getVoltage(ADCHANDLE, VoltageEnum selection, double* value);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_ADC_H_ */

