/************************************************
*
*  CROSSCONTROL AUX API
*
*  Backlight.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Backlight settings 
*
*  Use the function GetBacklight() to get a handle to pass on to the other functions. 
*  Use the method Backlight_release(BACKLIGHTHANDLE) to return the handle.
*
*  Example Usage:
*  @include backlight_example.cpp
*
************************************************/

#ifndef CC_BACKLIGHT_H_
#define CC_BACKLIGHT_H_

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* BACKLIGHTHANDLE;

  /*!
  * Factory function that creates instances of the Backlight object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return BACKLIGHTHANDLE to an allocated Backlight object. The returned
  * handle needs to be deallocated using the Backlight_release(BACKLIGHTHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling GetBacklight
  */
  EXTERN_C
    CCAUXDLL_API
    BACKLIGHTHANDLE
    CCAUXDLL_CALLING_CONV
    GetBacklight(void);

  /*!
  * Delete the backlight object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling Backlight_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Backlight_release(BACKLIGHTHANDLE);

  /*!
  * Get backlight intensity. 
  * Note that there might be hardware limitations, limiting the minimum and/or maximum value to other than (1..255).
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param intensity The current backlight intensity (1..255).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling Backlight_getIntensity
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getIntensity (BACKLIGHTHANDLE, unsigned char* intensity);

  /*!
  * Set backlight intensity. 
  * Note that there might be hardware limitations, limiting the minimum and/or maximum value to other than (1..255).
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param intensity The backlight intensity to set (1..255).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling Backlight_setIntensity
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_setIntensity(BACKLIGHTHANDLE, unsigned char intensity);

  /*!
  * Get backlight controller status.
  * Deprecated, use Backlight_getHWStatus instead.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Backlight controller status. Bit 0: status controller 1. Bit 1: status controller 2. Bit 2: status controller 3. Bit 3: status controller 4. 1=normal, 0=fault.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling Backlight_getStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getStatus(BACKLIGHTHANDLE, unsigned char* status);

  /*!
  * Get backlight hardware status.
  *
  * @param status Backlight controller status.
  * true: All backlight drivers works ok, false: one or more backlight drivers are faulty.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet backlight_example.cpp Calling Backlight_getHWStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getHWStatus(BACKLIGHTHANDLE, bool* status);

  /*!
  * Start automatic backlight control.
  * Note that reading the light sensor at the same time as running the automatic backlight control is not supported.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_startAutomaticBL(BACKLIGHTHANDLE);

  /*!
  * Stop automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_stopAutomaticBL(BACKLIGHTHANDLE);

  /*!
  * Get status from automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status 1=running, 0=stopped.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getAutomaticBLStatus(BACKLIGHTHANDLE, unsigned char* status);

  /*!
  * Set parameters for automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param bSoftTransitions Use soft transitions?
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_setAutomaticBLParams(BACKLIGHTHANDLE, bool bSoftTransitions);

  /*!
  * Get parameters for automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param bSoftTransitions Soft transitions used?
  * @param k K value.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Backlight_getAutomaticBLParams(BACKLIGHTHANDLE, bool* bSoftTransitions, double* k);

  /*!
  * Set light sensor filter parameters for automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param averageWndSize The average window size in nr of samples.
  * @param rejectWndSize The reject window size in nr of samples.
  * @param rejectDeltaInLux The reject delta in lux.
  * @param mode The configured sampling mode.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_setAutomaticBLFilter(
    BACKLIGHTHANDLE, 
    unsigned long averageWndSize,
    unsigned long rejectWndSize,
    unsigned long rejectDeltaInLux,
    LightSensorSamplingMode mode);

  /*!
  * Get light sensor filter parameters for automatic backlight control.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param averageWndSize The average window size in nr of samples.
  * @param rejectWndSize The reject window size in nr of samples.
  * @param rejectDeltaInLux The reject delta in lux.
  * @param mode The configured sampling mode.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getAutomaticBLFilter(
    BACKLIGHTHANDLE,
    unsigned long* averageWndSize,
    unsigned long* rejectWndSize,
    unsigned long* rejectDeltaInLux,
    LightSensorSamplingMode* mode);

  /*!
  * Get the current setting for Led dimming. If enabled, the function automatically
  * dimms the LED according to the current backlight setting; Low backlight gives less bright LED.
  * This works with manual backlight setting and automatic backlight, but only if the led is set to pure red, green or blue color.
  * If another color is being used, this functionality must be implemented separately.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_getLedDimming(BACKLIGHTHANDLE, CCStatus* status);

  /*!
  * Enable/disable Led dimming. If enabled, the function automatically
  * dimms the LED according to the current backlight setting; Low backlight gives less bright LED.
  * This works with manual backlight setting and automatic backlight, but only if the led is set to pure red, green or blue color.
  * If another color is being used, this functionality must be implemented separately.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Backlight_setLedDimming(BACKLIGHTHANDLE, CCStatus status);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_BACKLIGHT_H_ */


