/************************************************
*
*  CROSSCONTROL AUX API
*
*  Buzzer.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Buzzer settings 
*
*  Use the function GetBuzzer() to get a handle to pass on to the other functions. 
*  Use the method Buzzer_release(BUZZERHANDLE) to return the handle.
*
*  Example Usage:
*  @include buzzer_example.cpp
*
************************************************/
#ifndef CC_BUZZER_H 
#define CC_BUZZER_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* BUZZERHANDLE;

  /*!
  * Factory function that creates instances of the Buzzer object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return BUZZERHANDLE to an allocated Buzzer object. The returned
  * handle needs to be deallocated using the Buzzer_release(BUZZERHANDLE) 
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling GetBuzzer
  */
  EXTERN_C
    CCAUXDLL_API
    BUZZERHANDLE
    CCAUXDLL_CALLING_CONV
    GetBuzzer(void);

  /*!
  * Delete the Buzzer object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling Buzzer_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV 
    Buzzer_release(BUZZERHANDLE);

  /*!
  * Get buzzer frequency.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param frequency Current frequency (700-10000 Hz).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getFrequency(BUZZERHANDLE, unsigned short* frequency);

  /*!
  * Get buzzer volume.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param volume Current volume (0-51).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling Buzzer_getVolume
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getVolume(BUZZERHANDLE, unsigned short* volume);

  /*!
  * Get buzzer trigger. The Buzzer is enabled when the trigger is enabled.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param trigger Current trigger status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_getTrigger(BUZZERHANDLE, bool* trigger);

  /*!
  * Set buzzer frequency.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param frequency Frequency to set (700-10000 Hz).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling Buzzer_setFrequency
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setFrequency(BUZZERHANDLE, unsigned short frequency);

  /*!
  * Set buzzer volume.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param volume Volume to set (0-51).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling Buzzer_setVolume
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setVolume(BUZZERHANDLE, unsigned short volume);

  /*!
  * Set buzzer trigger. The Buzzer is enabled when the trigger is enabled.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param trigger Status to set.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_setTrigger(BUZZERHANDLE, bool trigger);

  /*!
  * Buzzes for a specified time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param time Time (ms) to buzz.
  * @param blocking Blocking or non-blocking function.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet buzzer_example.cpp Calling Buzzer_buzze
  */
  EXTERN_C
    CCAUXDLL_API
    eErr 
    CCAUXDLL_CALLING_CONV
    Buzzer_buzze(BUZZERHANDLE, int time, bool blocking);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_BUZZER_H */

