/************************************************
*
*  CROSSCONTROL AUX API
*
*  CfgIn.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Configurable inputs
*
*  Use the function GetCfgIn() to get a handle to pass on to the other functions.
*  Use the method CfgIn_release(CFGINHANDLE) to return the handle.
*
*  Example Usage:
*  @include cfgin_example.cpp
*
************************************************/
#ifndef CC_CFGIN_H
#define CC_CFGIN_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* CFGINHANDLE;

  /*!
  * Factory function that creates instances of the CfgIn object.
  *
  * Supported Platform(s): VC
  *
  * @return CFGINHANDLE to an allocated CfgIn object. The returned
  * handle needs to be deallocated using the CfgIn_release(CFGINHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling GetCfgIn
  */
  EXTERN_C
    CCAUXDLL_API
    CFGINHANDLE
    CCAUXDLL_CALLING_CONV
    GetCfgIn(void);

  /*!
  * Delete the CfgIn object.
  *
  * Supported Platform(s): VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling CfgIn_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    CfgIn_release(CFGINHANDLE);


  /*!
  * Set Configurable Input mode
  *
  * Supported Platform(s): VC
  *
  * @param channel Which configurable input channel to use, 1 or 2, corresponding to physical input channel
  * @param set_mode Which mode to set
  * Configurable input can be set to different measurement modes
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling CfgIn_setCfgInMode
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CfgIn_setCfgInMode (CFGINHANDLE, unsigned char channel, CfgInModeEnum set_mode);


  /*!
  * Get Configurable Input mode
  *
  * Supported Platform(s): VC
  *
  * @param channel Which configurable input channel to use, 1 or 2, corresponding to physical input channel
  * @param get_mode Storage container for retreived mode
  * Configurable input can be set to different measurement modes, this reads the setting back
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling CfgIn_getCfgInMode
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CfgIn_getCfgInMode (CFGINHANDLE, unsigned char channel, CfgInModeEnum* get_mode);

  /*!
  * Read the sampled value from configurable input, when in modes other than frequency mode:
  * CFGIN_HI_SWITCH    - sample_value is 0-1
  * CFGIN_LOW_SWITCH   - sample_value is 0-1
  * CFGIN_VOLTAGE_3V3  - sample_value is 0-33000 (0.1mV steps) 
  * CFGIN_VOLTAGE_5VPD - sample_value is 0-50000 (0.1mV steps) 
  * CFGIN_RESISTANCE   - sample_value is 0-65535 Ohm
  *
  * Supported Platform(s): VC
  *
  * @param channel Which configurable input channel to use, 1 or 2, corresponding to physical input channel
  * @param sample_value Read value which is relevant to actual mode setting
  * The actual value is dependent on the mode setting
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling CfgIn_getValue
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CfgIn_getValue (CFGINHANDLE, unsigned char channel, unsigned short* sample_value);


  /*!
  * Read the sampled value from configurable input, when in frequency measurement mode 
  * (CFGIN_FREQ_FLOATING, CFGIN_FREQ_PULLUP, CFGIN_FREQ_PULLDOWN).
  *
  * Supported Platform(s): VC
  *
  * @param channel Which configurable input channel to use, 1 or 2, corresponding to physical input channel
  * @param frequency Read signal frequency, 0.0 - 50000.0 Hz
  * @param duty_cycle Read signal duty cycle, 0-100%
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet cfgin_example.cpp Calling CfgIn_getPwmValue
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    CfgIn_getPwmValue (CFGINHANDLE, unsigned char channel, float* frequency, unsigned char* duty_cycle);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_CFGIN_H */

