/************************************************
*
*  CROSSCONTROL AUX API
*
*  Config.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Configuration of various settings
*
*  Use the function GetConfig() to get a handle to pass on to the other functions. 
*  Use the method Config_release(CONFIGHANDLE) to return the handle.
*
*  Example Usage:
*  @include config_example.cpp
*
************************************************/
#ifndef CC_CONFIG_H 
#define CC_CONFIG_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* CONFIGHANDLE;

  /*!
  * Bit defines for getVideoStartupPowerConfig and setVideoStartupPowerConfig
  */
  const unsigned char Video1Conf = (1 << 0); /*! Video channel 1 config */
  const unsigned char Video2Conf = (1 << 1); /*! Video channel 2 config */
  const unsigned char Video3Conf = (1 << 2); /*! Video channel 3 config */
  const unsigned char Video4Conf = (1 << 3); /*! Video channel 4 config */


  /*!
  * Factory function that creates instances of the Config object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return CONFIGHANDLE to an allocated Config object. The returned
  * handle needs to be deallocated using the Config_release(CONFIGHANDLE) 
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet config_example.cpp Calling GetConfig
  */
  EXTERN_C
    CCAUXDLL_API 
    CONFIGHANDLE
    CCAUXDLL_CALLING_CONV
    GetConfig();

  /*!
  * Delete the Config object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet config_example.cpp Calling Config_release    
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Config_release(CONFIGHANDLE);

  /*!
  * Get Start-up trigger configuration. Is the front button and/or the external on/off (ignition) signal
  * enabled as triggers for startup and wake up from suspended mode?
  * VC platform: CI state change and Can activity also available as wakeup triggers from suspend mode.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param config See enum TriggerConf.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet config_example.cpp Calling Config_getStartupTriggerConfig
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getStartupTriggerConfig(CONFIGHANDLE, TriggerConf* config);

  /*!
  * Get short button press action. Gets the configured action for a short button press:
  * NoAction, ActionSuspend or ActionShutDown.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The configured action.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getShortButtonPressAction(CONFIGHANDLE, PowerAction* action);

  /*!
  * Get long button press action. Gets the configured action for a long button press:
  * NoAction, ActionSuspend or ActionShutDown. A long button press is determined by the FrontBtnTrigTime.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The configured action.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getLongButtonPressAction(CONFIGHANDLE, PowerAction* action);

  /*!
  * Get On/Off signal action. Gets the configured action for an On/Off signal event:
  * NoAction, ActionSuspend or ActionShutDown. An On/Off signal event is determined by the ExtOnOffSigTrigTime.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The configured action.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getOnOffSigAction(CONFIGHANDLE, PowerAction* action);

  /*!
  * Get front button trigger time for long press.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param triggertime Time in milliseconds that the button has to
  * be pressed for the press to count as a long button press.
  * A button press twice this time will generate a hard shut down. If this time is set under 4000ms,
  * the hard shut down minimum time of 8s is used instead.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getFrontBtnTrigTime(CONFIGHANDLE, unsigned short* triggertime);

  /*!
  * Get external on/off signal trigger time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param triggertime Time in seconds that the external signal
  * has to be low for the unit to enter suspend mode or shut down (trigger an action).
  * This time can be set from one second up to several years, if needed.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getExtOnOffSigTrigTime(CONFIGHANDLE, unsigned long* triggertime);

  /*!
  * Get Button Function Configuration
  *
  * Supported Platform(s): VC
  *
  * @param button_number Which button to configure (1-MAX_BUTTONS)
  * @param button_config Bitfield for button configuration, see enum ButtonConfigEnum for details.
  * 
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet buttons_example.cpp Calling Config_getButtonFunction
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getButtonFunction(CONFIGHANDLE, unsigned char button_number, ButtonConfigEnum* button_config);

  /*!
  * Get suspend mode maximum time.
  *
  * Supported Platform(s): XL, XM, VC
  *
  * @param maxTime Maximum suspend time in minutes. After this time in
  * suspended mode, the unit will shut down to save power. A value of 0
  * means that the automatic shut down function is not used.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getSuspendMaxTime(CONFIGHANDLE, unsigned short* maxTime);

  /*!
  * Get Can power at startup configuration. The status of Can power
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setCanPowerStatus function.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getCanStartupPowerConfig(CONFIGHANDLE, CCStatus* status);

  /*!
  * Get Video power at startup configuration. The status of Video power
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setVideoPowerStatus function.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param config Bitwise representation of the four video channels. See the VideoXConf defines.
  * if the bit is 1, the power is enabled, else disabled.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getVideoStartupPowerConfig(CONFIGHANDLE, unsigned char* config);

  /*!
  * Get External fan power at startup configuration. The status
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setExtFanPowerStatus function.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getExtFanStartupPowerConfig(CONFIGHANDLE, CCStatus* status);

  /*!
  * Get the voltage threshold required for startup. The external voltage
  * must be stable above this value for the unit to start up.
  * The default and minimum value is 9V. It could be set to a higher
  * value for a 24V system.
  *
  * Supported Platform(s): XL, XM
  *
  * @param voltage The current voltage setting. (9V .. 28V)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getStartupVoltageConfig(CONFIGHANDLE, double* voltage);

  /*!
  * Get the temperature limit for heating. When temperature is below this limit,
  * the system is internally heated until the temperature rises above the limit.
  * The default and minimum value is -25 degrees Celsius. The maximum value is +5 degrees Celsius.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param temperature The current heating limit, in degrees Celsius (-25 to +5)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getHeatingTempLimit(CONFIGHANDLE, signed short* temperature);

  /*!
  * Get power on start-up behavior. If enabled, the unit always starts
  * when power is turned on, disregarding the setting for StartupTriggerConfig at that time.
  * The StartupTriggerConfig still applies if the unit is shut down or suspended, without
  * removing the power supply.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getPowerOnStartup(CONFIGHANDLE, CCStatus* status); 

  /*!
  * Set Start-up trigger configuration. Should the front button and/or the external on/off (ignition) signal
  * be enabled as triggers for startup and wake up from suspended mode?
  * VC platform: CI state change and Can activity also available as wakeup triggers from suspend mode.
  * 
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param conf See enum TriggerConf.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */	
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setStartupTriggerConfig(CONFIGHANDLE, TriggerConf conf);

  /*!
  * Set short button press action. Sets the configured action for a short button press:
  * NoAction, ActionSuspend or ActionShutDown.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The action to set.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet config_example.cpp Calling Config_setShortButtonPressAction
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setShortButtonPressAction(CONFIGHANDLE, PowerAction action);

  /*!
  * Set long button press action. Sets the configured action for a long button press:
  * NoAction, ActionSuspend or ActionShutDown. A long button press is determined by the FrontBtnTrigTime.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The action to set.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setLongButtonPressAction(CONFIGHANDLE, PowerAction action);

  /*!
  * Set On/Off signal action. Sets the configured action for an On/Off signal event:
  * NoAction, ActionSuspend or ActionShutDown. An On/Off signal event is determined by the ExtOnOffSigTrigTime.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param action The action to set.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setOnOffSigAction(CONFIGHANDLE, PowerAction action);

  /*!
  * Set front button trigger time for long press.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param triggertime Time in milliseconds that the button has to
  * be pressed for the press to count as a long button press.
  * A button press twice this time will generate a hard shut down. If this time is set under 4000ms,
  * the hard shut down minimum time of 8s is used instead.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setFrontBtnTrigTime(CONFIGHANDLE, unsigned short triggertime);

  /*!
  * Set external on/off signal trigger time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param triggertime Time in seconds that the external signal
  * has to be low for the unit to enter suspend mode or shut down (trigger an action).
  * This time can be set from one second up to several years, if needed.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setExtOnOffSigTrigTime(CONFIGHANDLE, unsigned long triggertime);

  /*!
  * Set button function configuration
  *
  * Supported Platform(s): VC
  *
  * @param button_number Which button to configure (1-MAX_BUTTONS)
  * @param button_config Bitfield for button configuration, see enum ButtonConfigEnum for details.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setButtonFunction(CONFIGHANDLE, unsigned char button_number, ButtonConfigEnum button_config);

  /*!
  * Set suspend mode maximum time.
  *
  * Supported Platform(s): XL, XM, VC
  *
  * @param maxTime Maximum suspend time in minutes. After this time in
  * suspended mode, the unit will shut down to save power. A value of 0
  * means that this function is not used.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setSuspendMaxTime(CONFIGHANDLE, unsigned short maxTime);

  /*!
  * Set Can power at startup configuration. The status of Can power
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setCanPowerStatus function.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setCanStartupPowerConfig(CONFIGHANDLE, CCStatus status);

  /*!
  * Set Video power at startup configuration. The status of Video power
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setVideoPowerStatus function.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param config Bitwise representation of the four video channels. See the VideoXConf defines.
  * if the bit is 1, the power is enabled, else disabled.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setVideoStartupPowerConfig(CONFIGHANDLE, unsigned char config);

  /*!
  * Set External fan power at startup configuration. The status
  * at startup and at resume from suspended mode. At resume
  * from suspend, this setting overrides the setting of the
  * setExtFanPowerStatus function.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Enabled/Disabled
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setExtFanStartupPowerConfig(CONFIGHANDLE, CCStatus status);

  /*!
  * Set the voltage threshold required for startup. The external voltage
  * must be stable above this value for the unit to start up.
  * The default and minimum value is 9V. It could be set to a higher
  * value for a 24V system.
  *
  * Supported Platform(s): XL, XM
  *
  * @param voltage The voltage to set (9V .. 28V).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setStartupVoltageConfig(CONFIGHANDLE, double voltage);

  /*!
  * Set the temperature limit for heating. When temperature is below this limit,
  * the system is internally heated until the temperature rises above the limit.
  * The default and minimum value is -25 degrees Celsius. The maximum value is +5 degrees Celsius.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param temperature The heating limit, in degrees Celsius (-25 to +5)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setHeatingTempLimit(CONFIGHANDLE, signed short temperature);

  /*!
  * Set power on start-up behavior. If enabled, the unit always starts
  * when power is turned on, disregarding the setting for StartupTriggerConfig at that time.
  * The StartupTriggerConfig still applies if the unit is shut down or suspended, without
  * removing the power supply.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status Enabled/Disabled
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setPowerOnStartup(CONFIGHANDLE, CCStatus status);

  /*!
  * Set RS485 mode enabled or disabled for RS4XX port.
  *
  * Supported Platform(s): XA, XS
  *
  * @param port RS4XX port (RS4XXPort1-4)
  * @param enabled RS485 enabled (true/false)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_setRS485Enabled (CONFIGHANDLE, RS4XXPort port, bool enabled);

  /*!
  * Get RS485 mode configuration for RS4XX port.
  *
  * Supported Platform(s): XA, XS
  *
  * @param port RS4XX port (RS4XXPort1-4)
  * @param enabled Is the RS485 port enabled (true/false)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Config_getRS485Enabled (CONFIGHANDLE, RS4XXPort port, bool* enabled);



#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_CONFIG_H */

