/************************************************
*
*  CROSSCONTROL AUX API
*
*  DigIO.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Read digital inputs
*
*  Use the function GetDigIO() to get a handle to pass on to the other functions. 
*  Use the method DigIO_release(DIGIOHANDLE) to return the handle.
*
*  Example Usage:
*  @include digio_example.cpp
*
************************************************/
#ifndef CC_DIGIO_H 
#define CC_DIGIO_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* DIGIOHANDLE;

  /*!
  * Bit defines for getDigIO
  */
  const unsigned char DigitalIn_1 = (1 << 0);
  const unsigned char DigitalIn_2 = (1 << 1);
  const unsigned char DigitalIn_3 = (1 << 2);
  const unsigned char DigitalIn_4 = (1 << 3);


  /*!
  * Factory function that creates instances of the DigIO object.  
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @return DIGIOHANDLE to an allocated DigIO object. The returned
  * handle needs to be deallocated using the DigIO_release(DIGIOHANDLE) 
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling GetDigIO
  */
  EXTERN_C
    CCAUXDLL_API
    DIGIOHANDLE
    CCAUXDLL_CALLING_CONV
    GetDigIO(void);

  /*!
  * Delete the DigIO object.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @return -
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling DigIO_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    DigIO_release(DIGIOHANDLE);

  /*!
  * Get Digital inputs.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Status of the four digital input pins.
  * Bit0: Digital input 1.
  * Bit1: Digital input 2.
  * Bit2: Digital input 3.
  * Bit3: Digital input 4.
  * Bit 4..7 are always zero.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling DigIO_getDigIO
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_getDigIO (DIGIOHANDLE, unsigned char* status);

  /*!
  * Set Digital outputs.
  *
  * Supported Platform(s): XA, XS  
  *
  * @param state State of the four digital output pins.
  * Bit0: Digital output 1.
  * Bit1: Digital output 2.
  * Bit2: Digital output 3.
  * Bit3: Digital output 4.
  * Bit 4..7 not used.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet digio_example.cpp Calling DigIO_setDigIO
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    DigIO_setDigIO (DIGIOHANDLE, unsigned char state);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_DIGIO_H */

