/************************************************
*
*  CROSSCONTROL AUX API
*
*  FirmwareUpgrade.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Firmware upgrade of the system's microprocessors and FPGA.
*
*  Use the function GetFirmwareUpgrade() to get a handle to pass on to the other functions. 
*  Use the method FirmwareUpgrade_release(FIRMWAREUPGHANDLE) to return the handle.
*
*  XA platform: Note that if you do several upgrades/verifications in a row, 
*  the FirmwareUpgrade object should be released and reinitialised between 
*  each operation: 
*  FirmwareUpgrade_release(pObj); pObj = ::GetFirmwareUpgrade();
*
*  Example Usage:
*  @include firmwareupgrade_example.cpp
************************************************/

#ifndef CC_FIRMWAREUPG_H_
#define CC_FIRMWAREUPG_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"


#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* FIRMWAREUPGHANDLE;

  /*!
  * Factory function that creates instances of the FirmwareUpgrade object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return FIRMWAREUPGHANDLE to an allocated FirmwareUpgrade object. The returned
  * handle needs to be deallocated using the FirmwareUpgrade_release(FIRMWAREUPGHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling GetFirmwareUpgrade
  */
  EXTERN_C
    CCAUXDLL_API
    FIRMWAREUPGHANDLE
    CCAUXDLL_CALLING_CONV
    GetFirmwareUpgrade(void);


  /*!
  * Delete the FirmwareUpgrade object.  
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_release(FIRMWAREUPGHANDLE);


  /*!
  * Start an upgrade of the FPGA.
  * After a FPGA upgrade, the system should be shut down. Full functionality 
  * of the system cannot be guaranteed until a fresh startup has been 
  * performed.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param filename Path and filename to the .mcs file to program.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call getUpgradeStatus to get the status
  * of the upgrade operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startFpgaUpgrade
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startFpgaUpgrade(FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Start a verification of the FPGA.
  * Verifies the FPGA against the file to program.
  * This could be useful if verification during programming fails.
  *
  * Supported Platform(s): XL, XM, XS, XA 
  *
  * @param filename Path and filename to the .mcs file to verify against.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call getUpgradeStatus to get the status
  * of the operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startFpgaVerification
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startFpgaVerification (FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Start an upgrade of the System Supervisor microprocessor (SS).
  * After an SS upgrade, the system must be shut down. The SS handles
  * functions for shutting down of the computer. In order to shut down after 
  * an upgrade, shut down the OS and then toggle the power. The backlight 
  * will still be on after the OS has shut down.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param filename Path and filename to the .hex file to program.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call fpgaUpgradeStatus to get the status
  * of the upgrade operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startSSUpgrade
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startSSUpgrade(FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Start a verification of the System Supervisor microprocessor (SS).
  * Verifies the SS against the file to program.
  * This could be useful if verification during programming fails.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param filename Path and filename to the .hex file to verify against.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call getUpgradeStatus to get the status
  * of the operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startSSVerification
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startSSVerification (FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Start an upgrade of the front microprocessor.
  * After a front upgrade, the system should be shut down.
  * The front will not work until a fresh startup has been performed.
  *
  * Supported Platform(s): XL, XM, XS, XA  
  *
  * @param filename Path and filename to the .hex file to program.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call fpgaUpgradeStatus to get the status
  * of the upgrade operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startFrontUpgrade
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startFrontUpgrade(FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Start a verification of the front microprocessor.
  * Verifies the front microprocessor against the file to program.
  * This could be useful if verification during programming fails.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param filename Path and filename to the .hex file to verify against.
  * @param blocking Whether or not the function should wait until
  * completion. If blocking is set to false, the function will return
  * immediately. One must then call getUpgradeStatus to get the status
  * of the operation. If blocking is set to true, the function
  * will return when the operation is complete. This might take a few
  * minutes.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet firmwareupgrade_example.cpp Calling FirmwareUpgrade_startFrontVerification
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_startFrontVerification (FIRMWAREUPGHANDLE, const char* filename, bool blocking);

  /*!
  * Gets the status of an upgrade operation. The upgrade status is common 
  * for all upgrade and verification methods.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC  
  *
  * @param status The current status of the upgrade operation.
  * @param blocking Whether or not the function should wait until a new
  * status event has been reported. If blocking is set to false, the
  * function will return immediately with the current status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_getUpgradeStatus(FIRMWAREUPGHANDLE, UpgradeStatus* status, bool blocking);

  /*!
  * Shut down the operating system.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FirmwareUpgrade_shutDown(FIRMWAREUPGHANDLE);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_FIRMWAREUPG_H_ */


