/************************************************
*
*  CROSSCONTROL AUX API
*
*  FrontLED.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Front LED control
* 
*  CCpilot VC platform - Button backlight control.
*  For VC, the API works the same way as for front LED control on other platforms, but without color support.
*
*  Use the function GetFrontLED() to get a handle to pass on to the other functions. 
*  Use the method FrontLED_release(FRONTLEDHANDLE) to return the handle.
*
*  Example Usage:
*  @include frontled_example.cpp
*
************************************************/
#ifndef CC_FRONT_LED_H 
#define CC_FRONT_LED_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* FRONTLEDHANDLE;


  /*!
  * Factory function that creates instances of the FrontLED object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return FRONTLEDHANDLE to an allocated FrontLED object. The returned
  * handle needs to be deallocated using the FrontLED_release(FRONTLEDHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling GetFrontLED
  */
  EXTERN_C
    CCAUXDLL_API
    FRONTLEDHANDLE
    CCAUXDLL_CALLING_CONV
    GetFrontLED(void);

  /*!
  * Delete the FrontLED object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_release    
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    FrontLED_release(FRONTLEDHANDLE);

  /*!
  * Get front LED signal. Note, the values may vary from previously set values with setSignal.
  * This is due to precision-loss in approximations.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_getSignal
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getSignal (FRONTLEDHANDLE, double* frequency, unsigned char* dutyCycle);

  /*!
  * Get front LED on time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param onTime Time in 10ms increments. 0 = off
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getOnTime (FRONTLEDHANDLE, unsigned char* onTime);

  /*!
  * Get front LED off time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param offTime Time in 10ms increments.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getOffTime (FRONTLEDHANDLE, unsigned char* offTime);

  /*!
  * Get front LED idle time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param idleTime Time in 100ms increments.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getIdleTime (FRONTLEDHANDLE, unsigned char* idleTime);

  /*!
  * Get number of pulses during a blink sequence.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param nrOfPulses Number of pulses.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getNrOfPulses (FRONTLEDHANDLE, unsigned char* nrOfPulses);

  /*!
  * Get front LED color mix.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  * On the VC platform - the blue parameter gets the button backlight intensity (0-15)
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_getColor
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getColor(FRONTLEDHANDLE, unsigned char* red, unsigned char* green, unsigned char* blue);

  /*!
  * Get front LED color from a set of standard colors. 
  * If the color is not one of the predefined colors, UNDEFINED_COLOR will be returned.
  * It is not recommended to use this function on the VC platform.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param color Color from CCAuxColor enum.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getStandardColor(FRONTLEDHANDLE, CCAuxColor* color);

  /*!
  * Is the front LED enabled during startup? If enabled, the LED will blink
  * yellow to indicate startup progress. It will turn green once the OS has started.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status LED Enabled or Disabled during startup.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_getEnabledDuringStartup(FRONTLEDHANDLE, CCStatus* status);

  /*!
  * Set front LED signal.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param frequency LED blink frequency (0.2-50 Hz).
  * @param dutyCycle LED on duty cycle (0-100%).
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_setSignal
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setSignal (FRONTLEDHANDLE, double frequency, unsigned char dutyCycle);

  /*!
  * Set front LED on time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param onTime Time in 10ms increments. 0 = off
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_setOnTime
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOnTime (FRONTLEDHANDLE, unsigned char onTime);

  /*!
  * Set front LED off time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param offTime Time in 10ms increments.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_setOffTime
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOffTime (FRONTLEDHANDLE, unsigned char offTime);

  /*!
  * Get front LED idle time.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param idleTime Time in 100ms.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setIdleTime (FRONTLEDHANDLE, unsigned char idleTime);

  /*!
  * Set front LED number of pulses during a blink sequence.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param nrOfPulses Number of pulses.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setNrOfPulses (FRONTLEDHANDLE, unsigned char nrOfPulses);

  /*!
  * Set front LED color mix.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  * On the VC platform - use the blue parameter to set the button backlight intensity (0-15)
  *
  * @param red Red color intensity 0-0x0F.
  * @param green Green color intensity 0-0x0F.
  * @param blue Blue color intensity 0-0x0F.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_setColor
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setColor(FRONTLEDHANDLE, unsigned char red, unsigned char green, unsigned char blue);

  /*!
  * Set one of the front LED standard colors.
  * It is not recommended to use this function on the VC platform.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param color Color from CCAuxColor enum.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet frontled_example.cpp Calling FrontLED_setStandardColor
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setStandardColor(FRONTLEDHANDLE, CCAuxColor color);

  /*!
  * Set front LED off.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setOff(FRONTLEDHANDLE);

  /*!
  * Should the front LED be enabled during startup? If enabled, the LED will blink
  * yellow to indicate startup progress. It will turn green once the OS has started.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status Enable or Disable the LED during startup.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    FrontLED_setEnabledDuringStartup(FRONTLEDHANDLE, CCStatus status);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_FRONT_LED_H */

