/************************************************
*
*  CROSSCONTROL AUX API
*
*  PWMOut.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  PWM Output
*
*  Use the function GetPWMOut() to get a handle to pass on to the other functions.
*  Use the method PWMOut_release(PWMOUTHANDLE) to return the handle.
*
*  Note: The rise and fall times of the hardware limits the signal's usefulness in high frequencies.
*  The duty cycle of the PWM signal affects the maximum attainable frequency: at 50% duty cycle, the attainable frequecy is at it's maximum.
*  If the duty cycle is near 0 or 100%, the rise/fall times of the signal means that the maximum attainable frequency is significantly lower.
*  A PWM signal is generated when both the frequency and the duty cycle of a channel is not zero.
*
*  Example Usage:
*  @include pwmout_example.cpp
*
************************************************/
#ifndef CC_PWMOUT_H
#define CC_PWMOUT_H

#include "CCAuxErrors.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* PWMOUTHANDLE;

  /*!
  * Factory function that creates instances of the PWMOut object.
  *
  * Supported Platform(s): VC
  *
  * @return PWMOUTHANDLE to an allocated PWMOut object. The returned
  * handle needs to be deallocated using the PWMOut_release(PWMOUTHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling GetPWMOut
  */
  EXTERN_C
    CCAUXDLL_API
    PWMOUTHANDLE
    CCAUXDLL_CALLING_CONV
    GetPWMOut(void);

  /*!
  * Delete the PWMOut object.
  *
  * Supported Platform(s): VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    PWMOut_release(PWMOUTHANDLE);


  /*!
  * Set PWM Output Duty cycle for a channel
  *
  * Supported Platform(s): VC
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param duty_cycle Which duty cycle (0-100 %) to use
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutputChannelDutyCycle
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutputChannelDutyCycle (PWMOUTHANDLE, unsigned char channel, unsigned char duty_cycle);


  /*!
  * Set PWM Output frequency for a channel
  *
  * Supported Platform(s): VC
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param frequency 0.0 - 5000.0 Hz frequency value
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutputChannelFrequency
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutputChannelFrequency (PWMOUTHANDLE, unsigned char channel, float frequency);


  /*!
  * Get PWM Output channel duty cycle
  *
  * Supported Platform(s): VC
  *
  * @param channel Which channel to get value from
  * There are two output channels, 1 or 2.
  * @param duty_cycle The read back duty cycle value
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputChannelDutyCycle
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputChannelDutyCycle (PWMOUTHANDLE, unsigned char channel, unsigned char* duty_cycle);


  /*!
  * Get PWM Output frequency for a channel
  *
  * Supported Platform(s): VC
  *
  * @param channel Which channel to set
  * There are two output channels, 1 or 2.
  * @param frequency 0.0 - 5000.0 Hz frequency value
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputChannelFrequency
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputChannelFrequency (PWMOUTHANDLE, unsigned char channel, float* frequency);


  /*!
  * Get PWM Output status
  *
  * Supported Platform(s): VC
  *
  * @param status Read back status value
  * Bit 0 represents PWM Output channel 1.
  * Bit 1 represents PWM Output channel 2.
  * If bit is set, it means short to ground or over temperature detected.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_getPWMOutputStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_getPWMOutputStatus (PWMOUTHANDLE, unsigned char* status);


  /*!
  * Turn off a PWM Output channel.
  * This function sets both frequency and duty cycle to 0. 
  *
  * Supported Platform(s): VC
  *
  * @param channel Which channel to set
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet pwmout_example.cpp Calling PWMOut_setPWMOutOff
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    PWMOut_setPWMOutOff (PWMOUTHANDLE, unsigned char channel);


#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_PWMOUT_H */

