/************************************************
*
*  CROSSCONTROL AUX API
*
*  Power.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: 
*  Power control access functions
*
*  Use the function GetPower() to get a handle to pass on to the other functions. 
*  Use the method Power_release(POWERHANDLE) to return the handle.
*
*  Example Usage:
*  @include power_example.cpp
*
************************************************/
#ifndef CC_POWER_H 
#define CC_POWER_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"


#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* POWERHANDLE;


  /*!
  * Factory function that creates instances of the Power object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return POWERHANDLE to an allocated Power object. The returned
  * handle needs to be deallocated using the Power_release(POWERHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet power_example.cpp Calling GetPower
  */
  EXTERN_C
    CCAUXDLL_API
    POWERHANDLE
    CCAUXDLL_CALLING_CONV
    GetPower(void);

  /*!
  * Delete the Power object.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return -
  *
  * Example Usage:
  * @snippet power_example.cpp Calling Power_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Power_release(POWERHANDLE);

  /*!
  * Get backlight power status.
  *
  * Supported Platform(s): XL, XM  
  *
  * @param status Backlight power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet power_example.cpp Calling Power_getBLPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getBLPowerStatus(POWERHANDLE, CCStatus* status);

  /*!
  * Get can power status.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Can power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getCanPowerStatus(POWERHANDLE, CCStatus* status);

  /*!
  * Get Video power status.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param videoStatus Video power status. Bit0: Video 1. Bit1: Video 2. Bit2: Video 3. Bit3: Video 4. (1=on, 0=off)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet power_example.cpp Calling Power_getVideoPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getVideoPowerStatus(POWERHANDLE, unsigned char* videoStatus);

  /*!
  * Get external fan power status.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Fan power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getExtFanPowerStatus(POWERHANDLE, CCStatus* status);

  /*!
  * Get the current status for front panel button and on/off signal.
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @param status The current status. See the definition of ButtonPowerTransitionStatus for details.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getButtonPowerTransitionStatus(POWERHANDLE, ButtonPowerTransitionStatus* status);

  /*!
  * Get Video power overcurrent detection status.
  * Find out if the video power supervision has detected overcurrent, likely caused by short circuit problems.
  * The overcurrent detection system will immediately turn of the power if such a condition occurs. 
  * If the overcurrent remains, video power is turned off permanently until the unit is restarted.  
  * Up to 5 consecutive over-current conditions needed until power is turned off completely.
  * If application software turns off and on the power, the failure counter will be reset.
  * 
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status The current overcurrent detection status
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  * 
  * Example Usage:
  * @snippet power_example.cpp Calling Power_getVideoOCDStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getVideoOCDStatus(POWERHANDLE, OCDStatus* status);

  /*!
  * Get Can power overcurrent detection status.
  * Find out if the Can power supervision has detected overcurrent, likely caused by short circuit problems.
  * The overcurrent detection system will immediately turn of the power if such a condition occurs. 
  * If the overcurrent remains, Can power is turned off permanently until the unit is restarted.  
  * Up to 5 consecutive over-current conditions needed until power is turned off completely.
  * If application software turns off and on the power, the failure counter will be reset.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status The current overcurrent detection status
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  * 
  * Example Usage:
  * @snippet power_example.cpp Calling Power_getCanOCDStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_getCanOCDStatus(POWERHANDLE, OCDStatus* status);

  /*!
  * Set backlight power status.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Backlight power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet power_example.cpp Calling Power_setBLPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_setBLPowerStatus(POWERHANDLE, CCStatus status);

  /*!
  * Set can power status.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Can power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_setCanPowerStatus(POWERHANDLE, CCStatus status);

  /*!
  * Set Video power status.
  *
  * Supported Platform(s): XL, XM, XS, XA
  *
  * @param status Video power status. 
  * Bit0: Video 1. 
  * Bit1: Video 2. 
  * Bit2: Video 3. 
  * Bit3: Video 4. (1=on, 0=off)
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_setVideoPowerStatus(POWERHANDLE, unsigned char status);

  /*!
  * Set external fan power status.
  *
  * Supported Platform(s): XL, XM
  *
  * @param status Fan power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_setExtFanPowerStatus(POWERHANDLE, CCStatus status);

  /*!
  * Acknowledge a power request from the system supervisor.
  * This is handled by the service/daemon and should normally not be used by applications
  * unless the CrossControl service/daemon is not being run on the system. If that is the case,
  * the following requests (read by getButtonPowerTransitionStatus) should be acknowledged:
  * BPTS_ShutDown, BPTS_Suspend and BPTS_Restart
  *
  * Supported Platform(s): XL, XM, XS, XA, VC
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Power_ackPowerRequest(POWERHANDLE);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_POWER_H */

