/************************************************
*
*  CROSSCONTROL AUX API
*
*  Smart.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description: Get S.M.A.R.T. data and information from the 
*  secondary storage device (CF/SD/HDD) where the OS is installed.
*
************************************************/

#ifndef CC_SMART_H_
#define CC_SMART_H_

#include "CCAuxErrors.h"
#include "CCPlatform.h"
#include <time.h>

#ifdef __cplusplus
namespace CrossControl
{
#endif

  // Handle type. In C++ language the interface type is used.
  typedef void* SMARTHANDLE;

  /*!
   * Factory function that creates instances of the Smart object.
   *
   * Supported Platform(s): XL, XM  
   *
   * @return SMARTHANDLE to an allocated AuxVersion structure. The returned
   * handle needs to be deallocated using the Smart::Release() method when
   * it's no longer needed.
   * Returns NULL if it fails to allocate memory.
   *
   * Example Usage:
   * @snippet smart_example.cpp Calling GetSmart
   */
  EXTERN_C
    CCAUXDLL_API
    SMARTHANDLE
    CCAUXDLL_CALLING_CONV
    GetSmart(void);

    /*!
    * Delete the Smart object.
    *
    * Supported Platform(s): XL, XM
    *
    * @return -
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling Release
    */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Smart_release (SMARTHANDLE);

    /*!
    * Get remaining lifetime of the secondary storage device.
    *
    * Supported Platform(s): XL, XM   
    *
    * @param lifetimepercent The expected remaining lifetime (0..100%).
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getRemainingLifeTime
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getRemainingLifeTime (SMARTHANDLE, unsigned char* lifetimepercent);

    /*!
    * Get remaining lifetime of the second secondary storage device. Use this function to access the second 
    * card if the the device uses two cards. 
    *
    * Supported Platform(s): XL  
    *
    * @param lifetimepercent The expected remaining lifetime (0..100%).
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. 
    * ERR_CODE_NOT_EXIST if only one card is available on XL platform. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getRemainingLifeTime2
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getRemainingLifeTime2 (SMARTHANDLE, unsigned char* lifetimepercent);


    /*!
    * Get serial number of the secondary storage device.
    *
    * Supported Platform(s): XL, XM     
    *
    * @param buff Text output buffer.
    * @param len Maximum length of the output buffer.
    * If the actual length of the data is greater, an error will be returned.
    * At least an 21 bytes buffer size must be used since the serial number can be 20 bytes + trailing zero.
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getDeviceSerial
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getDeviceSerial (SMARTHANDLE, char* buff, int len);

   /*!
    * Get serial number of the second secondary storage device. Use this function to access the second 
    * card if the the device uses two cards. 
    *
    * Supported Platform(s): XL        
    *
    * @param buff Text output buffer.
    * @param len Maximum length of the output buffer.
    * If the actual length of the data is greater, an error will be returned.
    * At least an 21 bytes buffer size must be used since the serial number can be 20 bytes + trailing zero.
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. 
    * ERR_CODE_NOT_EXIST if only one card is available on XL platform. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getDeviceSerial2
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getDeviceSerial2 (SMARTHANDLE, char* buff, int len);

    /*!
    * Get the date/time when the SMART monitoring began for this storage device. 
    * This time is either when the card first was used or when the system software was updated 
    * to support S.M.A.R.T. monitoring for the first time. Logging of time is based on the local time
    * of the computer at the time of logging and may therefore not always be accurate.
    *
    * Supported Platform(s): XL, XM          
    *
    * @param time A 32bit time_t value representing the number of seconds 
    * elapsed since 00:00 hours, Jan 1, 1970 UTC.
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getInitialTime
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getInitialTime (SMARTHANDLE, time_t* time);

   /*!
    * Get the date/time when the SMART monitoring began for this storage device. 
    * This time is either when the card first was used or when the system software was updated 
    * to support S.M.A.R.T. monitoring for the first time. Logging of time is based on the local time
    * of the computer at the time of logging and may therefore not always be accurate.
    *
    * Use this function to access the second card if the the device uses two cards. 
    *
    * Supported Platform(s): XL        
    *
    * @param time A 32bit time_t value representing the number of seconds 
    * elapsed since 00:00 hours, Jan 1, 1970 UTC.
    *
    * @return error status. 0 = ERR_SUCCESS, otherwise error code. 
    * ERR_CODE_NOT_EXIST if only one card is available on XL platform. See the enum eErr for details.
    *
    * Example Usage:
    * @snippet smart_example.cpp Calling getInitialTime2
    */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Smart_getInitialTime2 (SMARTHANDLE, time_t* time);

#ifdef __cplusplus
}
#endif

#endif /* CC_SMART_H_ */

