/************************************************
*
*  CROSSCONTROL AUX API
*
*  Telematics.h
*
*  Version 2.7.2.0, 2014-08-25
*
*  (c) maximatecc 2013
*
*  File description:
*  Power control and status functions for the optional telematics add-on card
*
*  Use the function GetTelematics() to get a handle to pass on to the other functions. 
*  Use the method Telematics_release(TELEMATICSHANDLE) to return the handle.
*
*  Example Usage:
*  @include telematics_example.cpp
*
************************************************/

#ifndef CC_TELEMATICS_H
#define CC_TELEMATICS_H

#include "CCAuxErrors.h"
#include "CCAuxTypes.h"
#include "CCPlatform.h"

#ifdef __cplusplus
namespace CrossControl {
#endif

  typedef void* TELEMATICSHANDLE;


  /*!
  * Factory function that creates instances of the Telematics object.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @return TELEMATICSHANDLE to an allocated Telematics object. The returned
  * handle needs to be deallocated using the Telematics_release(TELEMATICSHANDLE)
  * method when it's no longer needed.
  * Returns NULL if it fails to allocate memory.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling GetTelematics
  */
  EXTERN_C
    CCAUXDLL_API
    TELEMATICSHANDLE
    CCAUXDLL_CALLING_CONV
    GetTelematics(void);

  /*!
  * Delete the Telematics object.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @return -
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_release
  */
  EXTERN_C
    CCAUXDLL_API
    void
    CCAUXDLL_CALLING_CONV
    Telematics_release(TELEMATICSHANDLE);

  /*!
  * Is a telematics add-on card installed?
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status Enabled if a telematics add-on card is installed, otherwise Disabled.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getTelematicsAvailable
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getTelematicsAvailable(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get GPRS power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPRS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getGPRSPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getGPRSPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get GPRS power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPRS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getGPRSStartUpPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getGPRSStartUpPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get WLAN power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status WLAN power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getWLANPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getWLANPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get WLAN power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status WLAN power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getWLANStartUpPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getWLANStartUpPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get Bluetooth power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status Bluetooth power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getBTPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getBTPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get Bluetooth power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status Bluetooth power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getBTStartUpPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getBTStartUpPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get GPS power status.
  * Note that it can take some time after calling setGPSPowerStatus before the status is reported correctly.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getGPSPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getGPSPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get GPS power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getGPSStartUpPowerStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getGPSStartUpPowerStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Get GPS antenna status.
  * Antenna open/short detection. The status is set to disabled if no antenna is present or a short is detected.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPS antenna power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  *
  * Example Usage:
  * @snippet telematics_example.cpp Calling Telematics_getGPSAntennaStatus
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_getGPSAntennaStatus(TELEMATICSHANDLE, CCStatus* status);

  /*!
  * Set GPRS modem power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPRS modem power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setGPRSPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set GPRS power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPRS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setGPRSStartUpPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set WLAN power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status WLAN power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setWLANPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set WLAN power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status WLAN power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setWLANStartUpPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set Bluetooth power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status Bluetooth power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setBTPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set Bluetooth power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status Bluetooth power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setBTStartUpPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set GPS power status.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setGPSPowerStatus(TELEMATICSHANDLE, CCStatus status);

  /*!
  * Set GPS power status at startup and at resume from suspended mode.
  *
  * Supported Platform(s): XM, XA, XS      
  *
  * @param status GPS power status.
  *
  * @return error status. 0 = ERR_SUCCESS, otherwise error code. See the enum eErr for details.
  */
  EXTERN_C
    CCAUXDLL_API
    eErr
    CCAUXDLL_CALLING_CONV
    Telematics_setGPSStartUpPowerStatus(TELEMATICSHANDLE, CCStatus status);

#ifdef __cplusplus
} /* namespace CrossControl */
#endif

#endif /* CC_TELEMATICS_H */

